//--------------------------------------------------------------------------------------------------
//
// Title       : ps2_keyboard
// Design      : ps2_interface
// Author      : 0saka
// Company     : 0sa
//
//-------------------------------------------------------------------------------------------------
//
// File        : ps2_keyboard.v
// Generated   : Mon Dec 29 20:50:04 2003
// From        : interface description file
// By          : Itf2Vhdl ver. 1.20
//
//--------------------------------------------------------------------------------------------------------------
//
// Description :
//
//-------------------------------------------------------------------------------------------------
`timescale 1ps / 1ps


module ps2_keyboard ( clk ,data_ready ,ps2_clk ,ps2_data ,reset_n , read , keycode,err );
	
	input clk ;
	input reset_n;
	input read;
	input ps2_clk;
	input ps2_data;
	output data_ready;
	output err;
	output [7:0] keycode;
	
	wire clk ;
	wire reset_n ;
	wire err;
	reg [7:0] keycode;
	
	wire ps2_clk ;
	wire ps2_data ;
	wire data_ready;
	
	reg ps_clk_d;
	reg ps_data_d;
	reg ps_clk_s;
	reg ps_data_s;
	
	reg [5:0] bit_cnt;
	reg [12:0] timer_cnt;
	reg parity_err;
	reg ss_bit_err;
	
	reg [10:0] data;
	
	reg [2:0] current_state;
	reg [2:0] next_state;
	reg current_state2;
	reg next_state2;
        reg ignore_akey;
	
	wire ps_clk_posedge;
	wire ps_clk_negedge;
	wire shift;
	wire reset_timer;
	wire reset_bit_cnt;
	wire timer_timeout;
	wire shift_done;
	wire output_strobe;
	
	
	`define	S_H	0
	`define S_L	1
	`define	S_H2L	2
	`define S_L2H	3
	
	`define S_READY_ACK	0
	`define S_READY	1
	
	`define	TIMER_120U_TERMINAL	3000
	`define	FRAME_BIT_NUM	11
	
	/*synchronizing asynchronous input signal from ps2 ports */
	always @(posedge clk or negedge reset_n) begin
			if (~reset_n) begin
					ps_clk_d <= 1'b1;
					ps_data_d <= 1'b1;
					ps_clk_s <= 1'b1;
					ps_data_s <= 1'b1;
				end
			else begin
					ps_clk_d <= ps2_clk;
					ps_data_d <= ps2_data;
					ps_clk_s <= ps_clk_d;
					ps_data_s <= ps_data_d;
				end
		end
	
	assign ps_clk_posedge = (~ps_clk_s) & ps_clk_d; //synchronous ps2_clk Riging Edge
	assign ps_clk_negedge = (~ps_clk_d) & ps_clk_s;	//synchronous ps2_clk Falling Edge
	
	//State register for synchronize ps/2 clock
	always @(posedge clk or negedge reset_n) begin
			if (~reset_n)
				current_state <= `S_H;
			else
				current_state <= next_state;
		end
	
	/*State Machine for synchronize ps/2 clock (Sampling)*/
	always @(current_state or ps_clk_posedge or ps_clk_negedge) begin
			case (current_state)
				`S_L: begin
						if(ps_clk_posedge)
							next_state <= `S_L2H;
						else
							next_state <= `S_L;
					end
				`S_L2H: begin
						next_state <= `S_H;
					end
				`S_H: begin
						if(ps_clk_negedge)
							next_state <= `S_H2L;
						else
							next_state <= `S_H;
					end
				`S_H2L: begin
						next_state <= `S_L;
					end
				default: next_state <= `S_H;
			endcase
		end
	
	assign shift = (current_state == `S_H2L);
	assign reset_timer = ((current_state == `S_H2L)|(current_state == `S_L2H));
	
	//state2 Register for Handshaking protocol
	always @(posedge clk or negedge reset_n) begin
			if(~reset_n)
				current_state2 <= `S_READY_ACK;
			else
				current_state2 <= next_state2;
		end
	
	//Bit counter (count 11 bit)
	always @(posedge clk) begin
			if((~reset_n) || (shift_done))
				bit_cnt <= 0; //Normal Reset condition
			else if(reset_bit_cnt)
				bit_cnt <= 0; // timer output : no clock signal more than 60 us
			else if(shift)
				bit_cnt <= bit_cnt + 1;
		end
	
	assign reset_bit_cnt = ((timer_timeout)&(current_state == `S_H)&(ps_clk_s));
	
	//120us timer
	always @(posedge clk) begin
			if(reset_timer)
				timer_cnt <= 0;
			else if(~timer_timeout)
				timer_cnt <= timer_cnt + 1;
		end
	
	assign timer_timeout = (timer_cnt == `TIMER_120U_TERMINAL);
	
	//Shift Register for SIPO Operation (11-bit length)
	always @(posedge clk or negedge reset_n) begin
			if(~reset_n)
				data <= 0;
			else if(shift)
				data <= {ps_data_s,data[`FRAME_BIT_NUM-1:1]};
		end
	
	assign shift_done = (bit_cnt == `FRAME_BIT_NUM);
	assign output_strobe = (shift_done);
	
	//latch the output signals (scan code data)
	always @(posedge clk or negedge reset_n) begin
			if(~reset_n) begin
					ignore_akey <= 0;
					keycode <= 0;
				end
			else if(output_strobe) begin
				if(data[8:1] == 8'hf0) begin
					keycode <= 0;
					ignore_akey <= 1;
				end
				else if(~ignore_akey) begin
//                                  if (data[7] == 0) begin
				    keycode <= data[8:1];
//                                  end
				end
				else begin
					ignore_akey <= 0;
					end
				end
		end
	
	//State Machine that implements for handshaking protocol
	always @(current_state2 or output_strobe or read) begin
			case (current_state2)
				`S_READY_ACK: begin
						//send data
						if(output_strobe)
							next_state2 <= `S_READY;
						else
							next_state2 <= `S_READY_ACK;
					end
				`S_READY: begin
						//wait until Host send 'read' signal
						if(read)
							next_state2 <= `S_READY_ACK;
						else
							next_state2 <= `S_READY;
					end	 
				default: begin
						next_state2 <= `S_READY_ACK;
					end
			endcase
		end
	
	//send Host that data is ready
	assign data_ready = (current_state2 == `S_READY);		
	
	//Error Check Part
	always @(data) begin
			//odd parity check 
			if((data[9]  == ~^data[8:1]))
				parity_err <= 0;
			else
				parity_err <= 1;
			//START and STOP bit check	
			if((data[0] == 0)&&(data[10] == 1))
				ss_bit_err <= 0;
			else
				ss_bit_err <= 1;
		end
	
	//if this stream occur odd parity error or SS_bit error, send err signal 
//	assign err = parity_err | ss_bit_err;
	assign err = 0;
	
endmodule
